{
Copyright (c) 2013 Yvon Massé
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"),
to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
}

unit Unit3;
{$MODE delphi}

interface

uses
  SysUtils, Classes, Forms, Dialogs, StdCtrls, Math;

type  { TForm3 }
  TForm3 = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    CmbSo: TComboBox;
    ChkHi: TCheckBox;
    ChkCo: TCheckBox;
    ChkHe: TCheckBox;
    EdHi: TEdit;
    EdCo: TEdit;
    EdHe: TEdit;
    UniHi: TComboBox;
    UniCo: TComboBox;
    BtCalcul: TButton;
    BtRaz: TButton;
    procedure ChkHiClick(Sender: TObject);
    procedure ChkCoClick(Sender: TObject);
    procedure ChkHeClick(Sender: TObject);
    procedure EdHiExit(Sender: TObject);
    procedure EdCoExit(Sender: TObject);
    procedure EdHeExit(Sender: TObject);
    procedure UniHiChange(Sender: TObject);
    procedure UniCoChange(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure BtRazClick(Sender: TObject);
    procedure BtCalculClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end; 

var
  Form3: TForm3;
  hr: Integer = 20;

implementation

uses Unit1;

{$R *.lfm} { TForm3 }

function hoVhv(ho: Real): Real;
begin
  Result := ho - 1/60/Tan(DegToRad(ho + 7.31/(ho + 4.41)));
end;

function hvVho(hv: Real): Real;
const
  epsi = 1e-8;
  ni = 5;
var
  n, ie: Integer;
  he, hs: Array[0..1] of Real;
begin
  he[0] := hv;
  hs[0] := hoVhv(he[0]) - hv;
  he[1] := he[0] - hs[0];
  hs[1] := hoVhv(he[1]) - hv;
  // Boucle d'itération
  for n := 0 to ni do
  begin
    Result := (he[0]*hs[1] - he[1]*hs[0])/(hs[1] - hs[0]);
//    ShowMessage(IntToStr(n) + #13#10
//              + FloatToStr(he[0]) + ' / ' + FloatToStr(Result - he[0]) + #13#10
//              + FloatToStr(he[1]) + ' / ' + FloatToStr(Result - he[1]) + #13#10
//              + FloatToStr(Result) + ' / ' + FloatToStr(hoVhv(Result)));
    if SameValue(Result, he[0], epsi) or SameValue(Result, he[1], epsi) then break;
    ie := 0;
    if Abs(Result - he[1]) > Abs(Result - he[0]) then ie := 1;
    he[ie] := Result; hs[ie] := hoVhv(he[ie]) - hv;
  end;
  if n = ni then ShowMessage('Valeur : nombre d''itération maxi');
end;

procedure TForm3.ChkHiClick(Sender: TObject);
begin
  ChkClick(12);
end;

procedure TForm3.ChkCoClick(Sender: TObject);
begin
  ChkClick(13);
end;

procedure TForm3.ChkHeClick(Sender: TObject);
begin
  ChkClick(14);
end;

procedure TForm3.EdHiExit(Sender: TObject);
begin
  EdExit(12);
end;

procedure TForm3.EdCoExit(Sender: TObject);
begin
  EdExit(13);
end;

procedure TForm3.EdHeExit(Sender: TObject);
begin
  EdExit(14);
end;

procedure TForm3.UniHiChange(Sender: TObject);
begin
  UniChange(12);
end;

procedure TForm3.UniCoChange(Sender: TObject);
begin
  UniChange(13);
end;

procedure TForm3.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState
  );
// Outil de mise au point pour afficher les valeurs calculées en pleine définition
begin
  if (Key = Ord('Y')) and (ssCtrl in Shift) then
  begin
    quits();        // Pour quitter un éventuel champ de saisie
    ShowMessagePos('Valeurs de Tbol[i] et Tvar[i, 0] :'
        + #13#10'12 : ' + BoolToStr(Tbol[12], True)+ ', ' + FloatToStr(Tvar[12, 0])
        + #13#10'13 : ' + BoolToStr(Tbol[13], True)+ ', ' + FloatToStr(Tvar[13, 0])
        + #13#10'14 : ' + BoolToStr(Tbol[14], True)+ ', ' + FloatToStr(Tvar[14, 0]),
        Left + Width, Top);
  end;
end;

procedure TForm3.BtCalculClick(Sender: TObject);
const
  dep = 0.02943;
var
  i, p2, cf: Integer;
  bds: Real;
begin
  quits();                // Pour quitter un éventuel champ de saisie
  case CmbSo.ItemIndex of // Bord ou centre du Soleil
    0: bds := 0;          // Centre
    1: bds := 16/60;      // Bord inférieur
    2: bds := -16/60;     // Bord supérieur
  end;
  p2 := 1;
  cf := 0;
  for i := 12 to 14 do    // Calcul de la configuration
  begin
    if Tbol[i] then cf := cf + p2;
    p2 := 2*p2;
  end;
  if Tbol2[hr] then cf := cf + p2;
  case cf of
    0, 1, 2, 3, 4, 5, 6, 8, 9, 10, 12:
      MessageDlgPos('Il faut 3 paramètres', mtError, [mbOK], 0, Left + xms, Top + yms)
    else begin
      for i := 12 to 14 do                // Décoche la case s'il n'y a pas de valeur saisie
        if LiC[i].Checked and not Tbol[i] then LiC[i].Checked := False;
      if LiC2[hr].Checked and not Tbol2[hr] then LiC2[hr].Checked := False;
      case cf of    // Traitement de la configuration
        14, 13: begin
          bds := hvVho(modr(hr) - bds) + dep*Sqrt(Tvar[14,0]);
          if bds < 0 then
            MessageDlgPos('Hauteur réelle non observable', mtError, [mbOK], 0, Left + xms, Top + yms)
          else
            case cf of
              14: begin   // Calcul de hi
                Tvar[12,0] := bds - Tvar[13, 0];
                afrt(12);
              end;
              13: begin   // Calcul de co
                Tvar[13, 0] := bds - Tvar[12, 0];
                afrt(13);
              end;
            end;
        end;
        11, 7:
          if Tvar[12, 0] +  Tvar[13, 0] < 0 then
            MessageDlgPos('Hauteur observée négative', mtError, [mbOK], 0, Left + xms, Top + yms)
          else
            case cf of
              11: begin   // Calcul de he
                Tvar[14,0] := Sqr((Tvar[12, 0] + Tvar[13, 0] - hvVho(modr(hr) - bds))/dep);
                afrt(14);
              end;
              7: begin    // Calcul de hr
                Tvar[hr,0] := hoVhv(Tvar[12, 0]) + Tvar[13, 0] - dep*Sqrt(Tvar[14,0]) + bds;
                afrt(hr);
              end;
            end;
        else
          MessageDlgPos('Seulement 3 paramètres', mtError, [mbOK], 0, Left + xms, Top + yms);
      end;
    end;
  end;
end;

procedure TForm3.BtRazClick(Sender: TObject);
var
  i: Integer;
begin
  quits();                // Pour quitter un éventuel champ de saisie
  for i := 12 to 14 do
    if not Tbol[i] then
    begin
      LiE[i].Text := '';
      LiE[i].Modified := False;
    end;
  if not Tbol2[hr] then
  begin
    LiE[hr].Text := '';
    LiE[hr].Modified := False;
    Tbol[hr] := False;
  end;
end;

procedure TForm3.FormCreate(Sender: TObject);
var
  i: Integer;
begin
  LiC[12] := ChkHi; LiC[13] := ChkCo; LiC[14] := ChkHe;
  LiE[12] := EdHi; LiE[13] := EdCo; LiE[14] := EdHe;
  LiB[12] := UniHi; LiB[13] := UniCo;
  for i := 12 to 14 do
    Tbol[i] := False;
  for i := 12 to 13 do
    Unt[i] := LiB[i].Text;
  Unt[14] := '°';  // En fait en mètre, mais pour obtenir directement la valeur dans Tvar[14, 0]
  {$IF DEFINED(LINUX) OR DEFINED(DARWIN)}
  // Ajustement de la taille des Edit pour afficher entièrement les valeurs
  Width := Width + aged;
  Label3.Left := Label3.Left + aged div 2;
  Label4.Left := Label4.Left + aged;
  Label5.Left := Label5.Left + aged;
  CmbSo.Width := CmbSo.Width + aged;
  BtCalcul.Width := BtCalcul.Width + aged;
  for i := 12 to 14 do
  begin
    LiE[i].Width := LiE[i].Width + aged;
    if i < 14 then LiB[i].Left := LiB[i].Left + aged;
  end;
  {$ENDIF}
  {$IFDEF DARWIN}
  // Ajustement de la taille des ComboBox pour afficher entièrement les unités
  Width := Width + agcb;
  Label4.Left := Label4.Left + agcb div 2;
  BtCalcul.Width := BtCalcul.Width + agcb;
  for i := 12 to 13 do
    LiB[i].Width := LiB[i].Width + agcb;
  {$ENDIF}
end;

end.

